#include "SineGenerator.h"
#include <stdio.h>
#include <stdlib.h> // for malloc and free
#include <string.h> // for memcpy
#include <math.h>

static	AudioDeviceID			outputDevice = kAudioDeviceUnknown;
static	AudioStreamBasicDescription	outputDescription;
static	UInt32				outputBufferSize;
static	float				freq;

OSStatus audioIOProc (AudioDeviceID		inDevice,
                      const AudioTimeStamp*	inNow,
                      const AudioBufferList*	inInputData,
                      const AudioTimeStamp*	inInputTime,
                      AudioBufferList*		outOutputData,
                      const AudioTimeStamp*	inOutputTime,
                      void*			inClientData)
{
    static	int where = 0;
    UInt32	bufferSize = outOutputData->mBuffers[0].mDataByteSize;
    UInt32	numChannels = outOutputData->mBuffers[0].mNumberChannels;
    UInt32	sampleCount = bufferSize / sizeof(float);
    Float64	sampleRate = outputDescription.mSampleRate;
    UInt32	repeatCount = sampleCount / numChannels;
    float	*buf = (float*) malloc(sampleCount * sizeof(float));
    float	amp = 0.75;
    int		i, channel;

    for ( i = 0; i < repeatCount; i++ )
        for( channel = 0; channel < numChannels; channel++ )
            buf[2*i+channel] = amp * sin(2 * M_PI * freq / sampleRate * (where+i));
    
    memcpy(outOutputData->mBuffers[0].mData, (void*)buf, bufferSize);

    where += sampleCount/2;
    free(buf);

    return (kAudioHardwareNoError);    
}

void setup(int freak)
{
    OSStatus			err = kAudioHardwareNoError;
    UInt32			propertySize;

    // get the default output device
    propertySize = sizeof(AudioDeviceID);
    err = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice, &propertySize, (void *) &outputDevice);
    if (err != kAudioHardwareNoError) return;

    // get the size of the buffer
    propertySize = sizeof(UInt32);
    err = AudioDeviceGetProperty(outputDevice, 0, false, kAudioDevicePropertyBufferSize, &propertySize, &outputBufferSize);
    if (err != kAudioHardwareNoError) return;

    // get the description of the audio device
    propertySize = sizeof(AudioStreamBasicDescription);
    err = AudioDeviceGetProperty(outputDevice, 0, false, kAudioDevicePropertyStreamFormat, &propertySize, &outputDescription);
    if (err != kAudioHardwareNoError) return;

    // change to supplied frequency or default
    freq = freak;

    // we want linear PCM i guess
    if (outputDescription.mFormatID != kAudioFormatLinearPCM) return;
}

void printConfig()
{
    // print configuration
    
    return;
}

void playSine()
{
    OSStatus err;
    
    err = AudioDeviceAddIOProc(outputDevice, audioIOProc, NULL);
    if(err != kAudioHardwareNoError) return;
    
    err = AudioDeviceStart(outputDevice, audioIOProc);
    if(err != kAudioHardwareNoError) return;
}

void stopSine()
{
    OSStatus err;
    
    err = AudioDeviceStop(outputDevice, audioIOProc);
    if(err != kAudioHardwareNoError) return;

    err = AudioDeviceRemoveIOProc(outputDevice, audioIOProc);
    if(err != kAudioHardwareNoError) return;
}