#include <Carbon/Carbon.h>
#import <Foundation/Foundation.h>
#import <CoreAudio/CoreAudio.h>
#include <AudioToolbox/AudioFile.h>
#include <CoreServices/CoreServices.h>
#import <unistd.h>

#define SAMPLES_PER_BUFFER (8*1024)
static	short		myBuffer[SAMPLES_PER_BUFFER];

OSStatus audioDeviceIOProc(AudioDeviceID inDevice,
                           const AudioTimeStamp *inNow,
                           const AudioBufferList *inInputData,
                           const AudioTimeStamp *inInputTime,
                           AudioBufferList *outOutputData,
                           const AudioTimeStamp *inOutputTime,
                           void *inClientData);

    static	AudioDeviceID	outputDeviceID;
    static	AudioStreamBasicDescription	outputStreamBasicDescription;
//    unsigned int filePlace;
    AudioFileID myFile;

int main (int argc, const char * argv[]) {
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

    FSRef ref;
    FILE *file;
    char *path;
    
    path = "/Volumes/My Precious/Users/gappin/Desktop/mySound.aifc";

    file = fopen(path, "r");
//    if (!file) {
//        perror(path);
//        return 21;
//    }

    OSStatus status;
    UInt32 propertySize, bufferByteCount;

    NSLog(@"***Device Information***\n");
    // get the default audio device
    propertySize = sizeof(outputDeviceID);
    status = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice, &propertySize, &outputDeviceID);
    if (status) {
        NSLog(@"AudioHardwareGetProperty returned %d", status);
        return 10;
    }

    if (outputDeviceID == kAudioDeviceUnknown) {
        NSLog(@"AudioHardwareGetProperty: outputDeviceID is kAudioDeviceUnknown");
        return 11;
    }

    // configure the output device
    propertySize = sizeof(bufferByteCount);
    bufferByteCount = SAMPLES_PER_BUFFER * sizeof(float);
    status = AudioDeviceSetProperty(outputDeviceID, NULL, 0, NO, kAudioDevicePropertyBufferSize, propertySize, &bufferByteCount);
    if (status) {
        NSLog(@"AudioDeviceSetProperty: returned %d when setting kAudioDevicePropertyBufferSize to %d", status, SAMPLES_PER_BUFFER);
        return 12;
    }
    
    // get the output device description
    propertySize = sizeof(outputStreamBasicDescription);
    status = AudioDeviceGetProperty(outputDeviceID, 0, NO, kAudioDevicePropertyStreamFormat, &propertySize, &outputStreamBasicDescription);
    if (status) {
        NSLog(@"AudioDeviceGetProperty: returned %d when getting kAudioDevicePropertyStreamFormat", status);
        return 13;
    }

    FSPathMakeRef((const UInt8 *)path, &ref, FALSE);
    AudioFileOpen(&ref, NULL, NULL, &myFile);
    printf("fileID:%i\n", (int)myFile);

//    AudioFileInitialize(&ref, 2, &outputStreamBasicDescription, NULL, &myFile);


    status = AudioDeviceAddIOProc(outputDeviceID, audioDeviceIOProc, NULL);
    if (status) {
        NSLog(@"AudioDeviceAddIOProc: returned %d", status);
        return 15;
    }

    status = AudioDeviceStart(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceStart: returned %d", status);
        return 16;
    }
    

    while(1){}



    
    [pool release];
    return 0;
}


OSStatus audioDeviceIOProc(AudioDeviceID inDevice,
                           const AudioTimeStamp *inNow,
                           const AudioBufferList *inInputData,
                           const AudioTimeStamp *inInputTime,
                           AudioBufferList *outOutputData,
                           const AudioTimeStamp *inOutputTime,
                           void *inClientData)
{
    unsigned int sampleIndex, sampleCount;
    float *outBuffer;
    float scale = (1.0f / SHRT_MAX);
    UInt32 theSize = SAMPLES_PER_BUFFER * sizeof(float);

    assert(outOutputData->mNumberBuffers == 1);
    assert(outOutputData->mBuffers[0].mNumberChannels == 2);
    assert(outOutputData->mBuffers[0].mDataByteSize == (SAMPLES_PER_BUFFER * sizeof(float)));

    AudioFileReadBytes(myFile , false, 0, &theSize, myBuffer); 

//    memcpy(outOutputData->mBuffers[0].mData, myBuffer, SAMPLES_PER_BUFFER);

           memcpy(outOutputData->mBuffers[0].mData, myBuffer, outOutputData->mBuffers[0].mDataByteSize);

    
    return (kAudioHardwareNoError);
}
