#include "Player.h"

#import "Player.h"

#define SAMPLES_PER_BUFFER (8*1024)
#define	kMaxStringSize	1024

static	short		samples[SAMPLES_PER_BUFFER];
// the IOProc needs to see this
FILE	*file;

static	AudioDeviceID	outputDeviceID;
static	AudioStreamBasicDescription	outputStreamBasicDescription;
void signal_handler(int sig);

int playAiffFile(char filename[]) {
    static struct sigaction act;
    int error;

    // trap SIGINT
    act.sa_handler = signal_handler;
    sigfillset(&(act.sa_mask));
    sigaction(SIGINT, &act, NULL);
    
    if (error = setupOutputDevice() > 0 )
        return error;
    if (error = findAudioData(filename) > 0 )
        return error;
    if (error = playAudio() > 0)
        return error;
    
    while(!feof(file)){
        usleep( 1 * 1000 * 1000 );
    }

    cleanUp();

    return 0;
}

int findAudioData(char filename[]) {
    NSString *path = [[NSString alloc] initWithCString:(const char *)filename];
    const char *pathStr;
    AIFFChunkHeader chunkHeader;
    AIFFGenericChunk chunk;
    AIFFSSNDData ssndData;

    pathStr = [path fileSystemRepresentation];
    file = fopen(pathStr, "r");
    if (!file) {
        perror(pathStr);
        return 21;
    }

    fread(&chunkHeader, 1, sizeof(chunkHeader), file);
    if (chunkHeader.chunkID != 'FORM') {
        fprintf(stderr, "chunkID is not 'FORM'\n");
        fclose(file);
        return 20;
    }

    if (chunkHeader.fileType != 'AIFC') {
        fprintf(stderr, "file format is not 'AIFC'\n");
        fclose(file);
        return 22;
    }

    // Skip up to the 'SSND' chunk, ignoring all the type, compression, format, chunks.  I _said_ this was lame!
    while (1) {
        fread(&chunk, 1, sizeof(chunk), file);
        if (chunk.chunkID == 'SSND')
            break;

        fprintf(stderr, "skipping chunk %c%c%c%c\n", (chunk.chunkID >> 24) & 0xff, (chunk.chunkID >> 16) & 0xff, (chunk.chunkID >> 8) & 0xff, (chunk.chunkID >> 0) & 0xff);

        // Skip the chunk data
        fseek(file, chunk.chunkSize, SEEK_CUR);
    }

    fprintf(stderr, "Found SSND, size = %d\n", chunk.chunkSize);
    fread(&ssndData, 1, sizeof(ssndData), file);
    fprintf(stderr, "offset = %d\n", ssndData.offset);
    fprintf(stderr, "blockSize = %d\n", ssndData.blockSize);
    
    return 0;
}

int playAudio(){
    OSStatus status;
    
    status = AudioDeviceAddIOProc(outputDeviceID, audioDeviceIOProc, NULL);
    if (status) {
        NSLog(@"AudioDeviceAddIOProc: returned %d", status);
        return 15;
    }

    status = AudioDeviceStart(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceStart: returned %d", status);
        return 16;
    }
    
    return 0;
}

int stopAudio() {
    OSStatus status;
    
    status = AudioDeviceStop(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceStop: returned %d", status);
        return 16;
    }

    status = AudioDeviceRemoveIOProc(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceRemoveIOProc: returned %d", status);
        return 15;
    }
    
    return 0;
}

int cleanUp() {
    
    stopAudio();
    fclose(file);
    
    return 0;
}

int setupOutputDevice () {
    OSStatus status;
    UInt32 propertySize, bufferByteCount, theUInt32;
    char stringie[kMaxStringSize];

    NSLog(@"***Device Information***\n");
    // get the default audio device
    propertySize = sizeof(outputDeviceID);
    status = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice, &propertySize, &outputDeviceID);
    if (status) {
        NSLog(@"AudioHardwareGetProperty returned %d", status);
        return 10;
    }

    if (outputDeviceID == kAudioDeviceUnknown) {
        NSLog(@"AudioHardwareGetProperty: outputDeviceID is kAudioDeviceUnknown");
        return 11;
    }

    // get the device name
    propertySize = kMaxStringSize;
    status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyDeviceName, &propertySize, stringie);
    if(status == 0)
        NSLog(@"Device Name:%s", stringie);
    else
        NSLog(@"Couldn't get the device name.");

    // get the manufacturer name
    propertySize = kMaxStringSize;
    status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyDeviceManufacturer, &propertySize, stringie);
    if(status == 0)
        NSLog(@"Device Manufacturer:%s", stringie);
    else
        NSLog(@"Couldn't get the manufacturer name.");

    // check if something is in the headphone jack
    if(AudioDeviceGetPropertyInfo(outputDeviceID, 0, 0, kAudioDevicePropertyJackIsConnected, NULL, NULL) == 0)
    {
        theUInt32 = 0;
        propertySize = sizeof(UInt32);
        status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyJackIsConnected, &propertySize, &theUInt32);
        if(status == 0)
        {
            if (theUInt32)
                NSLog(@"Output jack connected.");
            else
                NSLog(@"Output jack not connected.");
        }
        else
            NSLog(@"Couldn't get the whether or not the output jack is connected.");
    }

    // configure the output device
    propertySize = sizeof(bufferByteCount);
    bufferByteCount = SAMPLES_PER_BUFFER * sizeof(float);
    status = AudioDeviceSetProperty(outputDeviceID, NULL, 0, NO, kAudioDevicePropertyBufferSize, propertySize, &bufferByteCount);
    if (status) {
        NSLog(@"AudioDeviceSetProperty: returned %d when setting kAudioDevicePropertyBufferSize to %d", status, SAMPLES_PER_BUFFER);
        return 12;
    }

    // get the output device description
    propertySize = sizeof(outputStreamBasicDescription);
    status = AudioDeviceGetProperty(outputDeviceID, 0, NO, kAudioDevicePropertyStreamFormat, &propertySize, &outputStreamBasicDescription);
    if (status) {
        NSLog(@"AudioDeviceGetProperty: returned %d when getting kAudioDevicePropertyStreamFormat", status);
        return 13;
    }

    // print the description
    NSLog(@"***Hardware Format***\n");
    NSLog(@"Sample Rate:%f", outputStreamBasicDescription.mSampleRate);
    NSLog(@"FormatID:%c%c%c%c",
          (outputStreamBasicDescription.mFormatID & 0xff000000) >> 24,
          (outputStreamBasicDescription.mFormatID & 0x00ff0000) >> 16,
          (outputStreamBasicDescription.mFormatID & 0x0000ff00) >>  8,
          (outputStreamBasicDescription.mFormatID & 0x000000ff) >>  0);
    NSLog(@"Bytes Per Frame:%d", outputStreamBasicDescription.mBytesPerFrame);
    NSLog(@"Channels Per Frame:%d", outputStreamBasicDescription.mChannelsPerFrame);
    NSLog(@"Bits Per Channel:%d", outputStreamBasicDescription.mBitsPerChannel);

    if (outputStreamBasicDescription.mFormatID != kAudioFormatLinearPCM) {
        NSLog(@"Default Audio Device doesn't support Linear PCM.");
        return 14;
    }

    return 0;
}

OSStatus audioDeviceIOProc(AudioDeviceID inDevice,
                           const AudioTimeStamp *inNow,
                           const AudioBufferList *inInputData,
                           const AudioTimeStamp *inInputTime,
                           AudioBufferList *outOutputData,
                           const AudioTimeStamp *inOutputTime,
                           void *inClientData)
{
    unsigned int sampleIndex, sampleCount;
    float *outBuffer;
    float scale = (1.0f / SHRT_MAX);

    assert(outOutputData->mNumberBuffers == 1);
    assert(outOutputData->mBuffers[0].mNumberChannels == 2);
    assert(outOutputData->mBuffers[0].mDataByteSize == (SAMPLES_PER_BUFFER * sizeof(float)));

    // The buffer that we need to fill
    outBuffer = (float *)outOutputData->mBuffers[0].mData;

    // Read some samples from the file.
    sampleCount = AIFFRead(file, samples, SAMPLES_PER_BUFFER);

    for (sampleIndex = 0; sampleIndex < sampleCount; sampleIndex++) {
        // Convert the samples from shorts to floats.  Scale the floats to be [-1..1].
        outBuffer[sampleIndex] = samples[sampleIndex] * scale;
    }

    // Fill in zeros in the rest of the buffer
    for (; sampleIndex < SAMPLES_PER_BUFFER; sampleIndex++)
        outBuffer[sampleIndex] = 0.0;

    return (kAudioHardwareNoError);
}

unsigned int AIFFRead(FILE *infile, short samples[], unsigned int sampleCount)
{
    unsigned int index;

    sampleCount = fread(samples, sizeof(short), sampleCount, infile);

    // CD data is stored in little-endian order, but we want big endian, being on PPC.
    for (index = 0; index < sampleCount; index++) {
        short sample;

        sample = samples[index];
        sample = ((sample & 0xff00) >> 8) | ((sample & 0x00ff) << 8);
        samples[index] = sample;
    }

    return sampleCount;
}

void signal_handler(int sig) {
    cleanUp();
    printf("\nGoodbye.\n");
    exit(0);
}