#import <Foundation/Foundation.h>
#import "Socket.h"
#import <unistd.h>
#import <CoreAudio/CoreAudio.h>

/*
Socket *socket;
NSMutableData *myBuffer;
NSString *myText;

int signal_handler(int sig);
int cleanUp();
 */

#define SAMPLES_PER_BUFFER (8*1024)
#define	kMaxStringSize	1024
static	short		samples[SAMPLES_PER_BUFFER];
static	AudioDeviceID	outputDeviceID;
static	AudioStreamBasicDescription	outputStreamBasicDescription;
static Socket *socket;


int tearDownIOProc ();
unsigned int giveMeAudioData ();
int putUpIOProc ();
OSStatus audioDeviceIOProc (AudioDeviceID inDevice,                            const AudioTimeStamp *inNow, const AudioBufferList *inInputData, const AudioTimeStamp *inInputTime, AudioBufferList *outOutputData, const AudioTimeStamp *inOutputTime, void *inClientData);
int setupOutputDevice ();

int main (int argc, const char * argv[]) {
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

//    Socket *socket;
//    NSString *myText;
    int port;
    port = 3030;
//    int stringLength;
//    NSString *host = [[NSString alloc] initWithCString:argv[1]];
    NSString *host = @"localhost";

    /* trap SIGINT
    static struct sigaction act;
    act.sa_handler = signal_handler;
    sigfillset(&(act.sa_mask));
    sigaction(SIGINT, &act, NULL);
    */
    
    socket = [[Socket alloc] init];
    [socket setReadBufferSize:SAMPLES_PER_BUFFER];
    setupOutputDevice();
    putUpIOProc();
    [socket connectToHostName:host port:port];

    while ([socket isConnected]) {
        usleep( 1 * 1000 * 1000 );
    }       
    
//    giveMeAudioData(socket);
    
    /*
    while ([socket isConnected]) {
        stringLength = [socket readData:myBuffer];
        usleep( 1 * 1000 * 1000 );
    }
    */

    //    cleanUp();
    tearDownIOProc();
    [socket close];
    [socket release];
    
    [pool release];
    return 0;
}

unsigned int giveMeAudioData () {
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    NSMutableData *myBuffer;
    NSString *myText;
    unsigned int bufferBytes;
    int index;
    myBuffer = [[NSMutableData alloc] initWithLength:SAMPLES_PER_BUFFER];
    
        bufferBytes = [socket readData:myBuffer];
        [myBuffer getBytes:samples length:SAMPLES_PER_BUFFER];
        myText = [[NSString alloc] initWithData:(NSData *)myBuffer  encoding:NULL];
        NSLog(@"Message Received:\n%@", myText);
        [myBuffer release];
        NSLog(@"Bytes read:%i", bufferBytes);

        for (index = 0; index < bufferBytes; index++) {
            short sample;

            sample = samples[index];
            sample = ((sample & 0xff00) >> 8) | ((sample & 0x00ff) << 8);
            samples[index] = sample;
        }        

        [pool release];
//        myText = [[NSString alloc] initWithData:(NSData *)myBuffer  encoding:NULL];
//        NSLog(@"Message Received:\n%@", myText);

    return bufferBytes;
}

int putUpIOProc () {
    OSStatus status;
    NSLog(@"in putUp");

    status = AudioDeviceAddIOProc(outputDeviceID, audioDeviceIOProc, NULL);
    if (status) {
        NSLog(@"AudioDeviceAddIOProc: returned %d", status);
        return 15;
    }

    status = AudioDeviceStart(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceStart: returned %d", status);
        return 16;
    }
    
    return 0;
}    

int tearDownIOProc () {
    OSStatus status;

    status = AudioDeviceStop(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceStop: returned %d", status);
        return 16;
    }

    status = AudioDeviceRemoveIOProc(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceRemoveIOProc: returned %d", status);
        return 15;
    }

    return 0;    
}

int setupOutputDevice () {
    OSStatus status;
    UInt32 propertySize, bufferByteCount, theUInt32;
    char stringie[kMaxStringSize];

    NSLog(@"***Device Information***\n");
    // get the default audio device
    propertySize = sizeof(outputDeviceID);
    status = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice, &propertySize, &outputDeviceID);
    if (status) {
        NSLog(@"AudioHardwareGetProperty returned %d", status);
        return 10;
    }

    if (outputDeviceID == kAudioDeviceUnknown) {
        NSLog(@"AudioHardwareGetProperty: outputDeviceID is kAudioDeviceUnknown");
        return 11;
    }

    // get the device name
    propertySize = kMaxStringSize;
    status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyDeviceName, &propertySize, stringie);
    if(status == 0)
        NSLog(@"Device Name:%s", stringie);
    else
        NSLog(@"Couldn't get the device name.");

    // get the manufacturer name
    propertySize = kMaxStringSize;
    status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyDeviceManufacturer, &propertySize, stringie);
    if(status == 0)
        NSLog(@"Device Manufacturer:%s", stringie);
    else
        NSLog(@"Couldn't get the manufacturer name.");

    // check if something is in the headphone jack
    if(AudioDeviceGetPropertyInfo(outputDeviceID, 0, 0, kAudioDevicePropertyJackIsConnected, NULL, NULL) == 0)
    {
        theUInt32 = 0;
        propertySize = sizeof(UInt32);
        status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyJackIsConnected, &propertySize, &theUInt32);
        if(status == 0)
        {
            if (theUInt32)
                NSLog(@"Output jack connected.");
            else
                NSLog(@"Output jack not connected.");
        }
        else
            NSLog(@"Couldn't get the whether or not the output jack is connected.");
    }

    // configure the output device
    propertySize = sizeof(bufferByteCount);
    bufferByteCount = SAMPLES_PER_BUFFER * sizeof(float);
    status = AudioDeviceSetProperty(outputDeviceID, NULL, 0, NO, kAudioDevicePropertyBufferSize, propertySize, &bufferByteCount);
    if (status) {
        NSLog(@"AudioDeviceSetProperty: returned %d when setting kAudioDevicePropertyBufferSize to %d", status, SAMPLES_PER_BUFFER);
        return 12;
    }

    // get the output device description
    propertySize = sizeof(outputStreamBasicDescription);
    status = AudioDeviceGetProperty(outputDeviceID, 0, NO, kAudioDevicePropertyStreamFormat, &propertySize, &outputStreamBasicDescription);
    if (status) {
        NSLog(@"AudioDeviceGetProperty: returned %d when getting kAudioDevicePropertyStreamFormat", status);
        return 13;
    }

    // print the description
    NSLog(@"***Hardware Format***\n");
    NSLog(@"Sample Rate:%f", outputStreamBasicDescription.mSampleRate);
    NSLog(@"FormatID:%c%c%c%c",
          (outputStreamBasicDescription.mFormatID & 0xff000000) >> 24,
          (outputStreamBasicDescription.mFormatID & 0x00ff0000) >> 16,
          (outputStreamBasicDescription.mFormatID & 0x0000ff00) >>  8,
          (outputStreamBasicDescription.mFormatID & 0x000000ff) >>  0);
    NSLog(@"Bytes Per Frame:%d", outputStreamBasicDescription.mBytesPerFrame);
    NSLog(@"Channels Per Frame:%d", outputStreamBasicDescription.mChannelsPerFrame);
    NSLog(@"Bits Per Channel:%d", outputStreamBasicDescription.mBitsPerChannel);

    if (outputStreamBasicDescription.mFormatID != kAudioFormatLinearPCM) {
        NSLog(@"Default Audio Device doesn't support Linear PCM.");
        return 14;
    }

    return 0;
}

OSStatus audioDeviceIOProc(AudioDeviceID inDevice,
                           const AudioTimeStamp *inNow,
                           const AudioBufferList *inInputData,
                           const AudioTimeStamp *inInputTime,
                           AudioBufferList *outOutputData,
                           const AudioTimeStamp *inOutputTime,
                           void *inClientData)
{
    unsigned int sampleIndex, sampleCount;
    float *outBuffer;
    float scale = (1.0f / SHRT_MAX);

    assert(outOutputData->mNumberBuffers == 1);
    assert(outOutputData->mBuffers[0].mNumberChannels == 2);
    assert(outOutputData->mBuffers[0].mDataByteSize == (SAMPLES_PER_BUFFER * sizeof(float)));

    // The buffer that we need to fill
    outBuffer = (float *)outOutputData->mBuffers[0].mData;

    // Read some samples from the file.
//    sampleCount = AIFFRead(file, samples, SAMPLES_PER_BUFFER);
    sampleCount = giveMeAudioData ();

    for (sampleIndex = 0; sampleIndex < sampleCount; sampleIndex++) {
        // Convert the samples from shorts to floats.  Scale the floats to be [-1..1].
        outBuffer[sampleIndex] = samples[sampleIndex] * scale;
    }

    // Fill in zeros in the rest of the buffer
    for (; sampleIndex < SAMPLES_PER_BUFFER; sampleIndex++)
        outBuffer[sampleIndex] = 0.0;

    return (kAudioHardwareNoError);
}






/*
int signal_handler(int sig) {
    cleanUp();
    NSLog(@"Goodbye.");
    exit(0);
}

int cleanUp() {
    [socket close];
    [socket release];
    [myBuffer release];
    [myText release];

    return 0;    
}
*/