#import <Foundation/Foundation.h>
#import "Socket.h"
#import <unistd.h>
#import <CoreAudio/CoreAudio.h>
#import "Server.h"

#define SAMPLES_PER_BUFFER (8*1024)
#define	kMaxStringSize	1024
static	short		samples[SAMPLES_PER_BUFFER];
FILE *file;

typedef struct _AIFFChunkHeader {
    unsigned int chunkID;
    unsigned int chunkSize;
    unsigned int fileType;
} AIFFChunkHeader;

typedef struct _AIFFGenericChunk {
    unsigned int chunkID;
    unsigned int chunkSize;
} AIFFGenericChunk;

typedef struct _AIFFSSNDData {
    unsigned int offset;
    unsigned int blockSize;
} AIFFSSNDData;


int main (int argc, const char * argv[]) {
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    Socket *socket;
    int port; port = 3030;
    NSData *transit;

    char *stringwoman = "/Volumes/My Precious/Users/gappin/Desktop/The Folder/Monica.aifc";
    findAudioData(stringwoman);

    socket = [[Socket alloc] init];
// necessary? it's writing, not reading
//    [socket setReadBufferSize:SAMPLES_PER_BUFFER];
    [socket listenOnPort:port];
    NSLog(@"Socket created. Listening on port:%i", port);
    [socket acceptConnection];
    NSLog(@"Connection established.");
    
    while (!feof(file) && [socket isConnected]) {
        AIFFRead(file, samples, SAMPLES_PER_BUFFER);

        // new
        transit = [[NSData alloc] initWithBytes:(void *)samples length:SAMPLES_PER_BUFFER];
        [socket writeData:transit];
        [transit release];
        
        usleep(5 * 1000 * 1000);
    }
    NSLog(@"Connection closed.");

    [socket close];
    [socket release];
    
    [pool release];
    return 0;
}

int findAudioData(char filename[]) {
    NSString *path = [[NSString alloc] initWithCString:(const char *)filename];
    const char *pathStr;
    AIFFChunkHeader chunkHeader;
    AIFFGenericChunk chunk;
    AIFFSSNDData ssndData;

    pathStr = [path fileSystemRepresentation];
    file = fopen(pathStr, "r");
    if (!file) {
        perror(pathStr);
        return 21;
    }

    fread(&chunkHeader, 1, sizeof(chunkHeader), file);
    if (chunkHeader.chunkID != 'FORM') {
        fprintf(stderr, "chunkID is not 'FORM'\n");
        fclose(file);
        return 20;
    }

    if (chunkHeader.fileType != 'AIFC') {
        fprintf(stderr, "file format is not 'AIFC'\n");
        fclose(file);
        return 22;
    }

    // Skip up to the 'SSND' chunk, ignoring all the type, compression, format, chunks.  I _said_ this was lame!
    while (1) {
        fread(&chunk, 1, sizeof(chunk), file);
        if (chunk.chunkID == 'SSND')
            break;

        fprintf(stderr, "skipping chunk %c%c%c%c\n", (chunk.chunkID >> 24) & 0xff, (chunk.chunkID >> 16) & 0xff, (chunk.chunkID >> 8) & 0xff, (chunk.chunkID >> 0) & 0xff);

        // Skip the chunk data
        fseek(file, chunk.chunkSize, SEEK_CUR);
    }

    fprintf(stderr, "Found SSND, size = %d\n", chunk.chunkSize);
    fread(&ssndData, 1, sizeof(ssndData), file);
    fprintf(stderr, "offset = %d\n", ssndData.offset);
    fprintf(stderr, "blockSize = %d\n", ssndData.blockSize);

    return 0;
}

unsigned int AIFFRead(FILE *file, short samples[], unsigned int sampleCount)
{
    unsigned int index;

    sampleCount = fread(samples, sizeof(short), sampleCount, file);

    // CD data is stored in little-endian order, but we want big endian, being on PPC.
    for (index = 0; index < sampleCount; index++) {
        short sample;

        sample = samples[index];
        sample = ((sample & 0xff00) >> 8) | ((sample & 0x00ff) << 8);
        samples[index] = sample;
    }

    return sampleCount;
}