#import "Client.h"

#define	kMaxStringSize	1024

static	AudioDeviceID	outputDeviceID;
static	AudioStreamBasicDescription	outputStreamBasicDescription;

/***
audioDeviceIOProc
***/

OSStatus audioDeviceIOProc(AudioDeviceID inDevice,
                           const AudioTimeStamp *inNow,
                           const AudioBufferList *inInputData,
                           const AudioTimeStamp *inInputTime,
                           AudioBufferList *outOutputData,
                           const AudioTimeStamp *inOutputTime,
                           void *inClientData)
{
    /***
    unsigned int sampleIndex, sampleCount;
    float *outBuffer;
    float scale = (1.0f / SHRT_MAX);

    assert(outOutputData->mNumberBuffers == 1);
    assert(outOutputData->mBuffers[0].mNumberChannels == 2);
    assert(outOutputData->mBuffers[0].mDataByteSize == (midSize * sizeof(float)));

    // the audio device buffer
    outBuffer = (float *)outOutputData->mBuffers[0].mData;

    // get some samples
    //sampleCount = AIFFRead(file, samples, SAMPLES_PER_BUFFER);
    sampleCount = getAudioData();

    for (sampleIndex = 0; sampleIndex < sampleCount; sampleIndex++) {
        // Convert the samples from shorts to floats.
        // Scale the floats to be [-1..1].
        outBuffer[sampleIndex] = samples[sampleIndex] * scale;
    }

    // Fill in zeros in the rest of the buffer
    for (; sampleIndex < SAMPLES_PER_BUFFER; sampleIndex++)
        outBuffer[sampleIndex] = 0.0;

    ***/
    return (kAudioHardwareNoError);
}

int getAudioData() {
    int samplesRead;

    return samplesRead;
}

/***
setupOutputDevice
***/
int setupOutputDevice() {
    OSStatus status;
    UInt32 propertySize, bufferByteCount;
    char stringie[kMaxStringSize];

    NSLog(@"***Device Information***\n");
    // get the default audio device
    propertySize = sizeof(outputDeviceID);
    status = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice, &propertySize, &outputDeviceID);
    if (status) {
        NSLog(@"AudioHardwareGetProperty returned %d", status);
        return 10;
    }

    if (outputDeviceID == kAudioDeviceUnknown) {
        NSLog(@"AudioHardwareGetProperty: outputDeviceID is kAudioDeviceUnknown");
        return 11;
    }

    // get the device name
    propertySize = kMaxStringSize;
    status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyDeviceName, &propertySize, stringie);
    if(status == 0)
        NSLog(@"Device Name:%s", stringie);
    else
        NSLog(@"Couldn't get the device name.");

    // get the manufacturer name
    propertySize = kMaxStringSize;
    status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyDeviceManufacturer, &propertySize, stringie);
    if(status == 0)
        NSLog(@"Device Manufacturer:%s", stringie);
    else
        NSLog(@"Couldn't get the manufacturer name.");

    /***
    // check if something is in the headphone jack
    if(AudioDeviceGetPropertyInfo(outputDeviceID, 0, 0, kAudioDevicePropertyJackIsConnected, NULL, NULL) == 0)
    {
        theUInt32 = 0;
        propertySize = sizeof(UInt32);
        status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyJackIsConnected, &propertySize, &theUInt32);
        if(status == 0)
        {
            if (theUInt32)
                NSLog(@"Output jack connected.");
            else
                NSLog(@"Output jack not connected.");
        }
        else
            NSLog(@"Couldn't get the whether or not the output jack is connected.");
    }
    ***/

    // configure the output device
    propertySize = sizeof(bufferByteCount);
    bufferByteCount = midSize * sizeof(float);
    status = AudioDeviceSetProperty(outputDeviceID, NULL, 0, NO, kAudioDevicePropertyBufferSize, propertySize, &bufferByteCount);
    if (status) {
        NSLog(@"AudioDeviceSetProperty: returned %d when setting kAudioDevicePropertyBufferSize to %d", status, midSize);
        return 12;
    }

    // get the output device description
    propertySize = sizeof(outputStreamBasicDescription);
    status = AudioDeviceGetProperty(outputDeviceID, 0, NO, kAudioDevicePropertyStreamFormat, &propertySize, &outputStreamBasicDescription);
    if (status) {
        NSLog(@"AudioDeviceGetProperty: returned %d when getting kAudioDevicePropertyStreamFormat", status);
        return 13;
    }

    // print the description
    NSLog(@"***Hardware Format***\n");
    NSLog(@"Sample Rate:%f", outputStreamBasicDescription.mSampleRate);
    NSLog(@"FormatID:%c%c%c%c",
          (outputStreamBasicDescription.mFormatID & 0xff000000) >> 24,
          (outputStreamBasicDescription.mFormatID & 0x00ff0000) >> 16,
          (outputStreamBasicDescription.mFormatID & 0x0000ff00) >>  8,
          (outputStreamBasicDescription.mFormatID & 0x000000ff) >>  0);
    NSLog(@"Bytes Per Frame:%d", outputStreamBasicDescription.mBytesPerFrame);
    NSLog(@"Channels Per Frame:%d", outputStreamBasicDescription.mChannelsPerFrame);
    NSLog(@"Bits Per Channel:%d", outputStreamBasicDescription.mBitsPerChannel);

    if (outputStreamBasicDescription.mFormatID != kAudioFormatLinearPCM) {
        NSLog(@"Default Audio Device doesn't support Linear PCM.");
        return 14;
    }

    return 0;
}

/***
startAudio
***/
int startAudio(){
    OSStatus status;

    status = AudioDeviceAddIOProc(outputDeviceID, audioDeviceIOProc, NULL);
    if (status) {
        NSLog(@"AudioDeviceAddIOProc: returned %d", status);
        return 15;
    }

    status = AudioDeviceStart(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceStart: returned %d", status);
        return 16;
    }

    return 0;
}

/***
stopAudio
***/
int stopAudio() {
    OSStatus status;

    status = AudioDeviceStop(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceStop: returned %d", status);
        return 16;
    }

    status = AudioDeviceRemoveIOProc(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceRemoveIOProc: returned %d", status);
        return 15;
    }

    return 0;
}