#import <Foundation/Foundation.h>
#import <CoreAudio/CoreAudio.h>
#import <unistd.h>
#import "Socket.h"

#define SAMPLES_PER_BUFFER (8*1024)
#define multiply 100
#define	kMaxStringSize	1024
#define showPtrs 1
#define preBuffer 1

Socket *mySocket;
//short mid[SAMPLES_PER_BUFFER];
short samples[multiply][SAMPLES_PER_BUFFER];
unsigned int inPt, outPt;

static	AudioDeviceID	outputDeviceID;
static	AudioStreamBasicDescription	outputStreamBasicDescription;

// prototypes
OSStatus audioDeviceIOProc(AudioDeviceID inDevice,
                           const AudioTimeStamp *inNow,
                           const AudioBufferList *inInputData,
                           const AudioTimeStamp *inInputTime,
                           AudioBufferList *outOutputData,
                           const AudioTimeStamp *inOutputTime,
                           void *inClientData);
int setupOutputDevice();
int startAudio();
int stopAudio();
int playAudio();
void signal_handler(int sig);


/***
This program reads buffers of audio data
from a socket and dsiplays them on the screen.
***/

int main (int argc, const char * argv[]) {
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

    int port; port = 3030;
    NSString *hostStr = [[NSString alloc] initWithCString:"localhost"];
//    NSString *hostStr = [[NSString alloc] initWithCString:"10.0.2.1"];
    static struct sigaction act;
    NSMutableData *caught;
    int bytesRead;

    // trap SIGINT
    act.sa_handler = *signal_handler;
    sigfillset(&(act.sa_mask));
    sigaction(SIGINT, &act, NULL);

    // set up buffer pointers
    inPt = 0;
    outPt = 0;

    int preBufferInt; preBufferInt = preBuffer;

    // setup audio device
    setupOutputDevice();
    
    // initialize socket
    mySocket = [[Socket alloc] init];
    [mySocket setReadBufferSize:(SAMPLES_PER_BUFFER * sizeof(short))];

    // connect to host
    [mySocket connectToHostName:hostStr port:port];
    [hostStr release];
    NSLog(@"Connected to host:%@", [mySocket remoteHostName]);

    // pre-buffer
    if ( preBufferInt == 1 ) {
        NSLog(@"Prebuffering.");
        do {
            caught = [[NSMutableData alloc] initWithCapacity:(SAMPLES_PER_BUFFER * sizeof(short))];
            
            // read data from the socket
            bytesRead = [mySocket readData:caught];
            NSLog(@"Bytes read from socket:%i", bytesRead);

            // fill in the rest of the buffer if needed
            if ( bytesRead < SAMPLES_PER_BUFFER ) {
                for (; bytesRead < SAMPLES_PER_BUFFER; bytesRead++)
                    samples[inPt][bytesRead] = 0;
            }

            // put data in buffer
            [caught getBytes:samples[inPt]];
            [caught release];
            inPt = ((inPt + 1) % multiply);
            if ( showPtrs ) {
                NSLog(@"inPt:%i", inPt);
            }
        } while ( inPt != 0 );
        NSLog(@"Prebuffering complete.");
    }
    
    playAudio();

    while ( [mySocket isConnected] ) {
//        if ( (((inPt+1)%multiply)!=outPt) && (outPt!=inPt) || preBufferInt == 0) {
        if ( (((inPt+1)%multiply)!=outPt) ) {
            // don't do another prebuffer check
            preBufferInt = 1;
            
            caught = [[NSMutableData alloc] initWithCapacity:(SAMPLES_PER_BUFFER * sizeof(short))];

            // read data from the socket
            bytesRead = [mySocket readData:caught];
            NSLog(@"Bytes read from socket:%i", bytesRead);
            
            // fill in the rest of the buffer if needed
            if ( bytesRead < SAMPLES_PER_BUFFER ) {
                for (; bytesRead < SAMPLES_PER_BUFFER; bytesRead++)
                    samples[inPt][bytesRead] = 0;
            }

            // put data in buffer
            [caught getBytes:samples[inPt]];

            /*
            int i;
            for ( i = 0; i < 100; i++)
                NSLog(@"samples[%i]:%i", i, samples[inPt][i]);
             */
            
            // release the old data structure
            [caught release];

            // increase buffer pointer
            inPt = ((inPt + 1) % multiply);
            if ( showPtrs ) {
                NSLog(@"inPt:%i", inPt);
                NSLog(@"outPt:%i", inPt);
            }
        }
        //usleep( 5 * 1000 * 1000 );
    }

    stopAudio();

    [mySocket release];
    NSLog(@"Normal termination.\nGoodbye.");
    
    [pool release];
    return 0;
}

void signal_handler(int sig) {
    NSLog(@"^C Handled.\nGoodbye.\n");
    if ( [mySocket isConnected] ) {
        [mySocket release];
    }
    exit(1);
}

/***
audioDeviceIOProc
***/

OSStatus audioDeviceIOProc(AudioDeviceID inDevice,
                           const AudioTimeStamp *inNow,
                           const AudioBufferList *inInputData,
                           const AudioTimeStamp *inInputTime,
                           AudioBufferList *outOutputData,
                           const AudioTimeStamp *inOutputTime,
                           void *inClientData)
{
    unsigned int sampleIndex;
    float *outBuffer;
    float scale = (1.0f / SHRT_MAX);

    assert(outOutputData->mNumberBuffers == 1);
    assert(outOutputData->mBuffers[0].mNumberChannels == 2);
    assert(outOutputData->mBuffers[0].mDataByteSize == (SAMPLES_PER_BUFFER * sizeof(float)));

    // the audio device buffer
    outBuffer = (float *)outOutputData->mBuffers[0].mData;

    // receive data from the buffer
    if ( ((outPt + 1) % multiply) != inPt) {
        for (sampleIndex = 0; sampleIndex < SAMPLES_PER_BUFFER; sampleIndex++) {
            outBuffer[sampleIndex] = samples[outPt][sampleIndex] * scale;
        }
        outPt = ((outPt + 1) % multiply);
    }
    else {
    /***
        sampleIndex = 0;
        for (; sampleIndex < SAMPLES_PER_BUFFER; sampleIndex++)
            outBuffer[sampleIndex] = (float)sampleIndex;
    ***/
    }
    
    return (kAudioHardwareNoError);
}

/***
setupOutputDevice
***/
int setupOutputDevice() {
    OSStatus status;
    UInt32 propertySize, bufferByteCount;
    char stringie[kMaxStringSize];

    NSLog(@"***Device Information***\n");
    // get the default audio device
    propertySize = sizeof(outputDeviceID);
    status = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice, &propertySize, &outputDeviceID);
    if (status) {
        NSLog(@"AudioHardwareGetProperty returned %d", status);
        return 10;
    }

    if (outputDeviceID == kAudioDeviceUnknown) {
        NSLog(@"AudioHardwareGetProperty: outputDeviceID is kAudioDeviceUnknown");
        return 11;
    }

    // get the device name
    propertySize = kMaxStringSize;
    status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyDeviceName, &propertySize, stringie);
    if(status == 0)
        NSLog(@"Device Name:%s", stringie);
    else
        NSLog(@"Couldn't get the device name.");

    // get the manufacturer name
    propertySize = kMaxStringSize;
    status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyDeviceManufacturer, &propertySize, stringie);
    if(status == 0)
        NSLog(@"Device Manufacturer:%s", stringie);
    else
        NSLog(@"Couldn't get the manufacturer name.");

    /***
        // check if something is in the headphone jack
        if(AudioDeviceGetPropertyInfo(outputDeviceID, 0, 0, kAudioDevicePropertyJackIsConnected, NULL, NULL) == 0)
    {
            theUInt32 = 0;
            propertySize = sizeof(UInt32);
            status = AudioDeviceGetProperty(outputDeviceID, 0, 0, kAudioDevicePropertyJackIsConnected, &propertySize, &theUInt32);
            if(status == 0)
            {
                if (theUInt32)
                    NSLog(@"Output jack connected.");
                else
                    NSLog(@"Output jack not connected.");
            }
            else
                NSLog(@"Couldn't get the whether or not the output jack is connected.");
    }
    ***/

    // configure the output device
    propertySize = sizeof(bufferByteCount);
    bufferByteCount = SAMPLES_PER_BUFFER * sizeof(float);
    status = AudioDeviceSetProperty(outputDeviceID, NULL, 0, NO, kAudioDevicePropertyBufferSize, propertySize, &bufferByteCount);
    if (status) {
        NSLog(@"AudioDeviceSetProperty: returned %d when setting kAudioDevicePropertyBufferSize to %d", status, SAMPLES_PER_BUFFER);
        return 12;
    }

    // get the output device description
    propertySize = sizeof(outputStreamBasicDescription);
    status = AudioDeviceGetProperty(outputDeviceID, 0, NO, kAudioDevicePropertyStreamFormat, &propertySize, &outputStreamBasicDescription);
    if (status) {
        NSLog(@"AudioDeviceGetProperty: returned %d when getting kAudioDevicePropertyStreamFormat", status);
        return 13;
    }

    // print the description
    NSLog(@"***Hardware Format***\n");
    NSLog(@"Sample Rate:%f", outputStreamBasicDescription.mSampleRate);
    NSLog(@"FormatID:%c%c%c%c",
          (outputStreamBasicDescription.mFormatID & 0xff000000) >> 24,
          (outputStreamBasicDescription.mFormatID & 0x00ff0000) >> 16,
          (outputStreamBasicDescription.mFormatID & 0x0000ff00) >>  8,
          (outputStreamBasicDescription.mFormatID & 0x000000ff) >>  0);
    NSLog(@"Bytes Per Frame:%d", outputStreamBasicDescription.mBytesPerFrame);
    NSLog(@"Channels Per Frame:%d", outputStreamBasicDescription.mChannelsPerFrame);
    NSLog(@"Bits Per Channel:%d", outputStreamBasicDescription.mBitsPerChannel);

    if (outputStreamBasicDescription.mFormatID != kAudioFormatLinearPCM) {
        NSLog(@"Default Audio Device doesn't support Linear PCM.");
        return 14;
    }

    return 0;
}

/***
startAudio
***/
int startAudio(){
    OSStatus status;

    status = AudioDeviceAddIOProc(outputDeviceID, audioDeviceIOProc, NULL);
    if (status) {
        NSLog(@"AudioDeviceAddIOProc: returned %d", status);
        return 15;
    }

    status = AudioDeviceStart(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceStart: returned %d", status);
        return 16;
    }

    return 0;
}

/***
stopAudio
***/
int stopAudio() {
    OSStatus status;

    status = AudioDeviceStop(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceStop: returned %d", status);
        return 16;
    }

    status = AudioDeviceRemoveIOProc(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceRemoveIOProc: returned %d", status);
        return 15;
    }

    return 0;
}

int playAudio() {
    OSStatus status;

    status = AudioDeviceAddIOProc(outputDeviceID, audioDeviceIOProc, NULL);
    if (status) {
        NSLog(@"AudioDeviceAddIOProc: returned %d", status);
        return 15;
    }
    
    status = AudioDeviceStart(outputDeviceID, audioDeviceIOProc);
    if (status) {
        NSLog(@"AudioDeviceStart: returned %d", status);
        return 16;
    }

    return 0;
}