#import "Server.h"


int findAudioData(FILE *file) {
    AIFFChunkHeader chunkHeader;
    AIFFGenericChunk chunk;
    AIFFSSNDData ssndData;

    fread(&chunkHeader, 1, sizeof(chunkHeader), file);
    if (chunkHeader.chunkID != 'FORM') {
        fprintf(stderr, "chunkID is not 'FORM'\n");
        fclose(file);
        return 20;
    }

    if (chunkHeader.fileType != 'AIFC') {
        fprintf(stderr, "file format is not 'AIFC'\n");
        fclose(file);
        return 22;
    }

    // Skip up to the 'SSND' chunk, ignoring all the type, compression, format, chunks.  I _said_ this was lame!
    while (1) {
        fread(&chunk, 1, sizeof(chunk), file);
        if (chunk.chunkID == 'SSND')
            break;

        fprintf(stderr, "skipping chunk %c%c%c%c\n", (chunk.chunkID >> 24) & 0xff, (chunk.chunkID >> 16) & 0xff, (chunk.chunkID >> 8) & 0xff, (chunk.chunkID >> 0) & 0xff);

        // Skip the chunk data
        fseek(file, chunk.chunkSize, SEEK_CUR);
    }

    fprintf(stderr, "Found SSND, size = %d\n", chunk.chunkSize);
    fread(&ssndData, 1, sizeof(ssndData), file);
    fprintf(stderr, "offset = %d\n", ssndData.offset);
    fprintf(stderr, "blockSize = %d\n", ssndData.blockSize);

    return 0;
}

unsigned int AIFFRead(FILE *file, short samples[], unsigned int sampleCount)
{
    unsigned int index;

    sampleCount = fread(samples, sizeof(short), sampleCount, file);

    // CD data is stored in little-endian order, but we want big endian, being on PPC.
    for (index = 0; index < sampleCount; index++) {
        short sample;

        sample = samples[index];
        sample = ((sample & 0xff00) >> 8) | ((sample & 0x00ff) << 8);
        samples[index] = sample;
    }

    return sampleCount;
}