#import <Foundation/Foundation.h>
#import <stdio.h>
#import <unistd.h>
#import "Socket.h"
#import "Server.h"

#define SAMPLES_PER_BUFFER (8*1024)
#define feederSleepTime 0.18

void signal_handler(int sig);

Socket *mySocket;

/***
This program opens a file, reads from it, and sends the
audio data down a socket to a client.
***/

/***
name:main

purpose:Read audio data from a file and send it down the socket.

input:a audio file

output:information about audio file and socket connection
***/
int main (int argc, const char * argv[]) {
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

    // declarations
    short samples[SAMPLES_PER_BUFFER];
    NSData *transit;
    int port; port = 3030;
    unsigned int bytesRead;
    NSString *path;
    const char *pathStr;
    const char *filePath = "/Volumes/My Precious/Users/gappin/Desktop/Jug Band Music.aifc";
    FILE *file;
    static struct sigaction act;

    // trap SIGINT
    act.sa_handler = *signal_handler;
    sigfillset(&(act.sa_mask));
    sigaction(SIGINT, &act, NULL);

    // open the file
    path = [[NSString alloc] initWithCString:filePath];
    pathStr = [path fileSystemRepresentation];
    file = fopen(pathStr, "r");
    if (!file) {
        perror(pathStr);
        return 21;
    }
    NSLog(@"Using file:%@", path);

    //check the file and search to audio data
    findAudioData(file);

    // initialize socket
    mySocket = [[Socket alloc] init];
    [mySocket listenOnPort:port];
    NSLog(@"Socket created. Listening on port:%i\n", port);
    [mySocket acceptConnection];
    NSLog(@"Connection established.\n");

    // while the file is not finished and the socket is up
    while (!feof(file) && [mySocket isConnected]) {
        // read from the file
        bytesRead = AIFFRead(file, samples, SAMPLES_PER_BUFFER);
        NSLog(@"%i bytes read and sent.", (bytesRead*sizeof(short)));

        // transmit the data
        transit = [[NSData alloc] initWithBytes:samples length:(SAMPLES_PER_BUFFER * sizeof(short))];

        //either wait to write to socket
        if ( [mySocket isWritable] )
            [mySocket writeData:transit];
        // or keep moving
        else
            NSLog(@"Data lost because socket not writable.");
        [transit release];

        NSLog(@"Sleeping %.2f seconds.", (float)feederSleepTime);
        usleep( feederSleepTime * 1000 * 1000 );
    }

    NSLog(@"Normal Termination.\nGoodbye.\n");

    [pool release];
    return 0;
}

void signal_handler(int sig) {
    NSLog(@"^C Handled.\nGoodbye.\n");
    if ( [mySocket isConnected] ) {
        [mySocket release];
    }
    exit(1);
}