#import "Controller.h"
#import <CoreAudio/AudioHardware.h>
#import <math.h>

OSStatus sineIOProc (AudioDeviceID inDevice, const AudioTimeStamp* inNow, const AudioBufferList* inInputData,
    const AudioTimeStamp* inInputTime, AudioBufferList*	outOutputData, const AudioTimeStamp* inOutputTime,
    void* inClientData)
{
    static int where = 0;
    Controller *controller = (Controller*) inClientData;
    int size = outOutputData->mBuffers[0].mDataByteSize;
    int sampleCount = size / sizeof(float);
    float *buf = (float*) malloc(sampleCount * sizeof(float));
    int i;
    float rate = [controller sampleRate];
    
    for(i=0; i<sampleCount/2; i++)
    {
        buf[2*i] = .75*sin(2*M_PI*440.0/rate*(where+i));
        buf[2*i+1] = .75*sin(2*M_PI*440.0/rate*(where+i)); // stereo
    }

    memcpy(outOutputData->mBuffers[0].mData, (void*)buf, size);	// move data 
    
    where += sampleCount/2;
    free(buf);
    
    return (noErr);     
}


@implementation Controller

-init
{
    _deviceReady = NO;
    _device = kAudioDeviceUnknown;
    _isPlaying = NO;
    return self;
}

-(void) awakeFromNib
{
    //NSLog(@"awaking...");
    [_mainWindow makeKeyAndOrderFront:self];
}

- (IBAction)playSine:(id)sender
{
    OSStatus		err = noErr;
   
    if(_isPlaying) return ;
    
    err = AudioDeviceAddIOProc(_device, sineIOProc, (void *) self);
    if(err != noErr) return;
    err = AudioDeviceStart(_device, sineIOProc);			
    if(err != noErr) return;
    
    _isPlaying = YES;

}

-(IBAction) stopPlaying:(id)sender
{
    OSStatus 	err = noErr;

    if(_isPlaying)
    {
        err = AudioDeviceStop(_device, sineIOProc);	
        if(err != noErr) return;
    
        err = AudioDeviceRemoveIOProc(_device, sineIOProc);	
        if(err != noErr) return;
        
        _isPlaying = false;
    
    }
}


-(void) applicationDidFinishLaunching:(NSNotification*) notification
{
    //NSLog(@"Did finish launching...");
    [self setupSoundDevice];
    
    NSLog(@"buffer size : %d", _bufferSize);
}

-(void) setupSoundDevice
{
    OSStatus				err = noErr;
    UInt32				count, bufferSize;
    AudioDeviceID			device = kAudioDeviceUnknown;
    AudioStreamBasicDescription		format;
    
    _deviceReady = NO;
    // get the default output device for the HAL
    count = sizeof(AudioDeviceID);
    err = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice,  &count, (void *) &device);
    if (err != noErr) return;
    
    // get the buffersize that the default device uses for IO
    count = sizeof(UInt32);
    err = AudioDeviceGetProperty(device, 0, false, kAudioDevicePropertyBufferSize, &count, &bufferSize);
    if (err != noErr) return;
   
    // get a description of the data format used by the default device
    count = sizeof(AudioStreamBasicDescription);
    err = AudioDeviceGetProperty(device, 0, false, kAudioDevicePropertyStreamFormat, &count, &format);
    if (err != noErr) return;
    
    // we want linear pcm
    if(format.mFormatID != kAudioFormatLinearPCM) return;

    // everything is ok so fill in these globals
    _device = device;
    _bufferSize = bufferSize;
    _format = format;
    
    _deviceReady = YES;
}
-(UInt32) bufferSize
{
    return _bufferSize;
}
-(double) sampleRate
{
    if(_deviceReady)
        return _format.mSampleRate;
    else return 44100.0;
}
@end
