//
//  main.m
//  PlayBufferedSoundFile
//
/*
 Copyright (c) 2002-2003, Kurt Revis.  All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of Snoize nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import <Foundation/Foundation.h>
#import "QTSoundFilePlayer.h"


int main(int argc, const char * argv[])
{
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    NSString *fileName = nil;
    QTSoundFilePlayer *player;

    // Check the arguments
    if (argc != 2) {
        fprintf(stderr, "Usage: %s soundFile\n", argv[0]);    
        exit(1);
    }
    fileName = [[[NSProcessInfo processInfo] arguments] objectAtIndex:1];

    // Is there a file present?
    if (![[NSFileManager defaultManager] isReadableFileAtPath:fileName]) {
        fprintf(stderr, "%s: '%s' could not be read\n", argv[0], argv[1]);
        exit(1);
    }

    // Create the player
    player = [[QTSoundFilePlayer alloc] initWithContentsOfFile:fileName];
    if (!player) {
        fprintf(stderr, "%s: we couldn't read the sound, or audio setup failed\n", argv[0]);
        exit(1);
    }
    
    // Play the sound
    if ([player play]) {
        // Now the sound is playing. You can do whatever you like here, but ideally you should go back to the
        // main run loop of your application. This is a command-line tool so we'll run the run loop manually.
        while ([player isPlaying])
            [[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.5]];        
    } else {
        fprintf(stderr, "%s: couldn't play the sound\n", argv[0]);
    }

    // We're done; clean up
    [player release];
    
    [pool release];
    return 0;
}
